#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const yargs_1 = __importDefault(require("yargs"));
const main_1 = require("./main");
const run = async () => {
    const argv = yargs_1.default
        .options({
        noImplicitAny: { type: 'boolean', default: true },
        noImplicitThis: { type: 'boolean', default: true },
        alwaysStrict: { type: 'boolean', default: true },
        strictBindCallApply: { type: 'boolean', default: true },
        strictNullChecks: { type: 'boolean', default: true },
        strictFunctionTypes: { type: 'boolean', default: true },
        strictPropertyInitialization: { type: 'boolean', default: true },
        noEmit: { type: 'boolean', default: true },
        targetBranch: { type: 'string', default: 'master' },
    })
        .parserConfiguration({
        'strip-dashed': true,
    }).argv;
    const typeScriptOptions = Object.entries(argv)
        .filter(([_, value]) => typeof value === 'boolean')
        .reduce((options, [key, value]) => Object.assign(Object.assign(Object.assign({}, options), { [key]: value })), {});
    const { targetBranch } = argv;
    const result = await main_1.strictify({
        targetBranch,
        typeScriptOptions,
        onFoundSinceRevision: (revision) => {
            revision
                ? console.log(`🔍  Finding changed files since ${chalk_1.default.bold('git')} revision ${chalk_1.default.bold(revision)}`)
                : console.log(`⚠️  Can not find commit at which the current branch was forked from ${chalk_1.default.bold(targetBranch)}. Does target branch ${chalk_1.default.bold(targetBranch)} exists?`);
        },
        onFoundChangedFiles: (changedFiles) => {
            console.log(`🎯  Found ${chalk_1.default.bold(String(changedFiles.length))} changed ${changedFiles.length === 1 ? 'file' : 'files'}`);
        },
        onExamineFile: (file) => {
            console.log(`🔍  Checking ${chalk_1.default.bold(file)} ...`);
        },
        onCheckFile: (file, hasError) => hasError
            ? console.log(`❌  ${chalk_1.default.bold(file)} failed`)
            : console.log(`✅  ${chalk_1.default.bold(file)} passed`),
    });
    if (result.errors) {
        process.exit(1);
    }
    else {
        console.log(`🎉  ${chalk_1.default.green('All files passed')}`);
    }
};
run();
