"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const promise_1 = __importDefault(require("simple-git/promise"));
const execa_1 = __importDefault(require("execa"));
const isSupportedExtension = (fileName) => Boolean(fileName.match(/\.tsx?$/));
const findCommitAtWhichTheCurrentBranchForkedFromTargetBranch = async (targetBranch) => {
    return execa_1.default('git', ['merge-base', '--fork-point', targetBranch])
        .then((resposne) => resposne.stdout)
        .catch(() => undefined);
};
const findModifiedAndUntrackedFiles = async () => {
    return promise_1.default()
        .status()
        .then(({ created, modified, not_added }) => [...created, ...modified, ...not_added])
        .catch((e) => {
        console.warn('Can not find modified and untracked files', e);
        return [];
    });
};
const findFilesFromDiffToRevision = async (revision) => {
    return revision
        ? promise_1.default()
            .diffSummary([revision])
            .then(({ files }) => files.reduce((result, { file }) => [...result, file], []))
            .catch((e) => {
            console.error('Can not find files that changed compared to master', e);
            return [];
        })
        : [];
};
const getTypeScriptCompileOutput = async (options) => {
    const args = Object.entries(options)
        .map(([key, value]) => [key.replace(/^/, '--'), value])
        .reduce((result, [key, value]) => [...result, key, value], []);
    let tscOutput = [];
    try {
        await execa_1.default('tsc', args);
    }
    catch (error) {
        const { all } = error;
        tscOutput = all.split('\n');
    }
    return tscOutput;
};
exports.strictify = async (args) => {
    const { onFoundSinceRevision, onFoundChangedFiles, onCheckFile, typeScriptOptions, targetBranch, } = args;
    const commit = await findCommitAtWhichTheCurrentBranchForkedFromTargetBranch(targetBranch);
    onFoundSinceRevision(commit);
    const changedFiles = await Promise.all([
        findModifiedAndUntrackedFiles(),
        findFilesFromDiffToRevision(commit),
    ]).then(([a, b]) => Array.from(new Set([...a, ...b])).filter(isSupportedExtension));
    onFoundChangedFiles(changedFiles);
    if (changedFiles.length === 0) {
        return { success: true, errors: 0 };
    }
    const tscOut = await getTypeScriptCompileOutput(typeScriptOptions);
    const errorCount = changedFiles.reduce((totalErrorCount, fileName) => {
        let errorCount = 0;
        tscOut.map((line) => {
            if (line.includes(fileName)) {
                errorCount === 0 ? onCheckFile(fileName, true) : null;
                totalErrorCount++;
                errorCount++;
                console.log(line);
            }
        });
        errorCount === 0 ? onCheckFile(fileName, false) : null;
        return totalErrorCount;
    }, 0);
    return {
        success: errorCount === 0,
        errors: errorCount,
    };
};
